/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
// var InAppBrowserOptions = 'location=no,zoom=no,toolbar=no'

var commands = {
  login: function(userId){
    console.log('user logged in as ' + userId)
    tagUser(userId)
  },
  logout: function(){
    console.log('user logged out')
    untagUser()
  },
  download: function(url){
    console.log('user wants to download ' + url)
    downloadImage(url)
  }
}

var createIframe = function(url){
  var iframe = document.createElement('iframe')
  iframe.width = '100%'
  iframe.height = '100%'
  iframe.src = url

  document.body.appendChild(iframe)

  window.addEventListener('message', function(event){
    console.log('received message')
    console.log(event)

    var command = event.data.command
    var arg = event.data.arg

    commands[command](arg)
  })

  iframe.addEventListener('load', function(){
    var target = iframe.contentWindow
    target.postMessage({command: 'set_app'}, '*')
    console.log('iframe loaded')
  })

  return iframe
}

var iframeEventsAttached = {
  message: false,
  load: false
}

var attachIframeEvents = function(iframe){

  if (!iframeEventsAttached.message)
    window.addEventListener('message', function(event){
      console.log('received message')
      console.log(event)

      var command = event.data.command
      var arg = event.data.arg

      commands[command](arg)
      iframeEventsAttached.message = true
    })

  if (!iframeEventsAttached.load)
    iframe.addEventListener('load', function(){
      var target = iframe.contentWindow
      target.postMessage({command: 'set_app'}, '*')
      console.log('iframe loaded')
      iframeEventsAttached.load = true
    })
}


var openInFrame = function(url){
  var iframe = document.querySelector('iframe.app_content')

  attachIframeEvents(iframe)

  iframe.src = url
}

var tagUser = function(userId){
  window.plugins.OneSignal.sendTag('user_id', userId)
}

var untagUser = function(){
  window.plugins.OneSignal.deleteTag('user_id')
}

var downloadImage = function(url){

  SpinnerDialog.show(null, 'Descargando')

  var path = cordova.file.externalRootDirectory
  var fileUrl = encodeURI(url)
  var photoName = url.split(/\//).pop()

  var fileTransfer = new FileTransfer()

  if (navigator.platform .match(/Linux/)) {
    window.resolveLocalFileSystemURL(cordova.file.externalRootDirectory, onFileSystemSuccess, onError)
  } else {
    // window.resolveLocalFileSystemURL(cordova.file.documentsDirectory, onFileSystemSuccess, onError)
    window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, onFileSystemSuccess, onError)
  }

  function onError(e) {
    navigator.notification.alert("No hay permiso para almacenar archivos", function(){}, 'Error')
  }

  function onFileSystemSuccess(fileSystem) {
    var entry = ""
    if (navigator.platform.match(/Linux/)) {
      entry = fileSystem
    } else {
      entry = fileSystem.root
    }
    entry.getDirectory("Download", {
      create: true,
      exclusive: false
    }, onGetDirectorySuccess, onGetDirectoryFail)
  }

  function onGetDirectoryFail(e){
    navigator.notification.alert("No se puede acceder a la carpeta de descargas", function(){}, 'Error')
  }

  function onGetDirectorySuccess(dir) {
    console.log(dir)
    cdr = dir
    dir.getFile(photoName, {
      create: true,
      exclusive: false
    }, gotFileEntry, errorHandler)
  }

  function errorHandler(e){
    navigator.notification.alert("No se puede acceder a la carpeta de descargas", function(){}, 'Error')
  }

  function mimeFor(extension){
    var mimes = {
      xls: 'application/vnd.ms-excel',
      xlsx: 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
      doc: 'application/msword',
      docx: 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
      pdf: 'application/pdf',
      jpg: 'image/jpeg',
      jpeg: 'image/jpeg',
      png: 'image/png',
      gif: 'image/gif'
    }

    return mimes[extension]
  }

  function gotFileEntry(fileEntry) {
    console.log(fileEntry)
    fileTransfer.download(fileUrl, fileEntry.toURL(),
      function(entry) {
        SpinnerDialog.hide()
        console.log(fileUrl)
        console.log(cdr.nativeURL)
        navigator.notification.confirm('Documento descargado', function(buttonIndex){
          if (buttonIndex == 2)
            return

          cordova.plugins.fileOpener2.open(
              fileEntry.toURL(),
              mimeFor(fileEntry.toURL().match(/\.[^\.]*?$/)[0].slice(1)),
              {
                  error : function(error){
                    navigator.notification.alert(JSON.stringify(error), function(){}, 'Error')
                  },
                  success : function(){ }
              }
          );
        }, 'HFE', ['Abrir', 'No'])

        // cordova.plugins.imagesaver.saveImageToGallery(fileEntry.toURL(), function(){
        //   navigator.notification.alert('Foto guardada', function(){}, 'Info')
        // }, function(){
        //   navigator.notification.alert('Error guardando foto', function(){}, 'Error')
        // })
      },
      function(error) {
        SpinnerDialog.hide();
        navigator.notification.alert(JSON.stringify(error), function(){}, 'Error')
      },
      false
    )
  }
  /****/
}

var app = {
    initialize: function() {
        document.addEventListener('deviceready', this.onDeviceReady.bind(this), false)
    },

    onDeviceReady: function() {
        this.setUpOneSignal()
        openInFrame('http://app.sentoentrenadorpersonal.com')
    },

    setUpOneSignal: function() {
      window.plugins.OneSignal.setLogLevel({logLevel: 4, visualLevel: 1})

      var notificationOpenedCallback = function(jsonData) {
        console.log('notificationOpenedCallback: ' + JSON.stringify(jsonData))
        console.log('navigating to: ' + jsonData.notification.payload.additionalData.url)
        // cordova.InAppBrowser.open(jsonData.notification.payload.additionalData.url, '_blank', InAppBrowserOptions)
        openInFrame(jsonData.notification.payload.additionalData.url)
      }

      var notificationReceivedCallback = function(jsonData) {
        console.log('notificationReceivedCallback: ' + JSON.stringify(jsonData))
      }

      window.plugins.OneSignal
        .startInit("b00a9072-5c3a-4ef8-84cb-828294344466")
        .handleNotificationOpened(notificationOpenedCallback)
        .handleNotificationReceived(notificationReceivedCallback)
        .inFocusDisplaying(window.plugins.OneSignal.OSInFocusDisplayOption.Notification)
        .endInit()
      // Call syncHashedEmail anywhere in your app if you have the user's email.
      // This improves the effectiveness of OneSignal's "best-time" notification scheduling feature.
      // window.plugins.OneSignal.syncHashedEmail(userEmail)
    }
}

app.initialize()
